@@ -1,6 +1,6 @@
 #!/bin/sh
 # autogen.sh - generates configure using the autotools
-# $Id: buildconf.sh,v 1.1.1.1 2005/03/13 17:02:36 konn Exp $
+# $Id: buildconf.sh,v 1.2 2006/06/09 20:46:29 konn Exp $
 #export WANT_AUTOCONF=2.5
 #libtoolize --force --copy
 rm -f ./aclocal.m4
@@ -13,7 +13,7 @@ rm -f ./config.sub
 rm -f ./config.status
 rm -f ./install-sh
 rm -f ./libtool
-rm -f ./ltmain.sh
+#rm -f ./ltmain.sh
 rm -f ./missing
 rm -f ./mkinstalldirs
 aclocal 
